#include "ns3/helper-module.h"
#include "ns3/simulator-module.h"
#include "ns3/core-module.h"
#include <fstream>

using namespace ns3;

int main (int argc, char *argv[])
{
  CommandLine cmd;
  cmd.Parse (argc, argv);

  NodeContainer nodes;
  nodes.Create (2);

  PointToPointHelper p2p;
  p2p.SetDeviceAttribute ("DataRate", StringValue ("5Mbps"));
  p2p.SetChannelAttribute ("Delay", StringValue ("2ms"));
  NetDeviceContainer devices = p2p.Install (nodes);

  InternetStackHelper stack;
  stack.Install (nodes);

  Ipv4AddressHelper ipv4;
  ipv4.SetBase ("10.1.1.0", "255.255.255.0");
  Ipv4InterfaceContainer addresses = ipv4.Assign (devices);

  Ipv4GlobalRoutingHelper::PopulateRoutingTables ();

  ProcessManagerHelper processManager;
  processManager.Install (nodes);

  ProcessHelper process;
  process.SetBinary ("ccnd");
  process.SetStackSize (1<<16);
  ApplicationContainer apps = process.Install (nodes);
  apps.Start (Seconds (1.0));

  Simulator::Run ();
  Simulator::Destroy ();

  return 0;
}
