#!/usr/bin/env python

import subprocess
import time
import math

class Result:
    def __init__(self):
        self.runs = 0
        self.min_time = 0
        self.max_time = 0
        self.avg_time = 0
        self.std_dev = 0

def bench (args, max_std_dev = 0.05, min_runs = 10, max_runs = 100, 
           print_individual = False, handleOutput=None):
    min_time = 1000000.0
    max_time = 0.0
    n = 0
    avg_time = 0.0
    square_sum = 0.0
    for runs in range(0,max_runs):
        start = time.time()
        prgm = subprocess.Popen(args, 
                                stdin = subprocess.PIPE, 
                                stdout = subprocess.PIPE,
                                stderr = subprocess.STDOUT)
        output = prgm.communicate(input=None)
        if handleOutput is not None:
            handleOutput (output)
        end = time.time()
        duration = end - start
        if print_individual:
            print duration
        min_time = min(min_time,duration)
        max_time = max(max_time,duration)
        n += 1
        delta = duration - avg_time
        avg_time += delta / n
        square_sum += delta * (duration - avg_time)
        if n >= min_runs and \
                math.sqrt (square_sum / (n - 1)) / avg_time < max_std_dev:
            break
    result = Result ()
    result.min_time = min_time
    result.max_time = max_time
    result.avg_time = avg_time
    result.runs = n
    result.std_dev = math.sqrt (square_sum / (n - 1))
    return result

if __name__ == '__main__':
    import optparse
    parser = optparse.OptionParser()
    parser.disable_interspersed_args()
    parser.add_option('--std-dev', type='float', dest='std_dev', default=0.05, 
                      help='The maximum standard deviation allowed.' \
                      ' The standard deviation is normalized to the mean so 0.05 means that we will keep doing'
                      'measurements until the standard deviation divided by the mean is lower than 0.05')
    parser.add_option('--min-runs', type='int', dest='min_runs', default=10,
                      help="The minimum number of runs performed until we start testing for ' \
                      ' conformance to the standard deviation requirement.")
    parser.add_option('--max-runs', type='int', dest='max_runs', default=100,
                      help="The maximum number of runs performed until we stop.")
    parser.add_option('--print-individual', action='store_true', dest='print_individual', default=False,
                      help="Output the timing information for each run on stdout, one line per run.")
    (options,args) = parser.parse_args()
    result = bench(args, options.std_dev, options.min_runs, options.max_runs, options.print_individual)
    print 'runs=' + str(result.runs) + ' min_time=' + str(result.min_time) + \
        ' max_time=' + str(result.max_time) + ' avg=' + str(result.avg_time) + \
        ' std_dev=' + str(result.std_dev)
