#!/usr/bin/env python

import re
import sys
import string

reg = re.compile('([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),([^,\n]+)$')

data_by_hops = dict()
data_by_rate = dict()

for l in open(sys.argv[1]).readlines()[1:]:
    match = reg.search (l)
    if match is not None:
        rate = match.group(1)
        nhops = match.group(2)
        bytes = match.group(3)
        bandwidth = match.group(4)
        runtime = match.group(5)
        mss = match.group(6)
        if rate not in data_by_rate.keys():
            data_by_rate[rate] = [[nhops,bytes,bandwidth,runtime,mss]]
        else:
            data_by_rate[rate].append([nhops,bytes,bandwidth,runtime,mss])
        if nhops not in data_by_hops.keys():
            data_by_hops[nhops] = [[rate,bandwidth,bytes,runtime,mss]]
        else:
            data_by_hops[nhops].append([rate,bandwidth,bytes,runtime,mss])


class Gnuplot:
    def __init__(self):
        pass
    def generate(self, script):
        import subprocess
        gnuplot = subprocess.Popen (['gnuplot', '-'], 
                                    stdin=subprocess.PIPE,
                                    stdout=subprocess.PIPE, 
                                    stderr=subprocess.STDOUT)
        (out,err) = gnuplot.communicate (script)

def plot_runtime_vs_iperf(data):
    str = """
set xlabel "Iperf bytes "
set ylabel "Simulation runtime (s)"
set title "Simulation runtime vs iperf bytes"
set terminal postscript
set output 'runtime-vs-bytes.ps'
"""
    datasets = []
    commands = []
    for k in data.keys():
        commands.append("'-' with lines title \"%s hops\""  % k)
        vector = [[c,d] for (a,b,c,d,e) in data[k]]
        datasets.append(vector)
    str += "plot "
    str += ",".join(commands)
    str += '\n'

    for set in datasets:
        for (a,b) in set:
            str+= a + ' ' + b + '\n'
        str += 'e\n'

    gnuplot = Gnuplot()
    gnuplot.generate(str)

def plot_runtime_bandwidth_vs_hops(data):
    import string
    for k in data.keys():
        datasets = []
        commands = []
        tmpl = string.Template ("""
set xlabel "Hops"
set ylabel "Iperf bytes"
set y2label "Simulation runtime (s)"
set ytics nomirror
set y2tics
set autoscale y2
set title "link=$throughput bytes/runtime vs hops"
set terminal postscript
set output 'bytes-runtime-vs-hops-$throughput.ps'
""")
        str = tmpl.substitute(throughput=k)
        commands.append("'-' with lines axis x1y1 title \"iperf bytes\"")
        commands.append("'-' with lines axis x1y2 title \"runtime (s)\"")
        v1 = [[a,c] for (a,b,c,d,e) in data[k]]
        v2 = [[a,d] for (a,b,c,d,e) in data[k]]
        datasets.append(v1)
        datasets.append(v2)
        str += "plot "
        str += ",".join(commands)
        str += '\n'

        for set in datasets:
            for (a,b) in set:
                str += a + ' ' + b + '\n'
            str += 'e\n'

        gnuplot = Gnuplot()
        gnuplot.generate(str)

plot_runtime_vs_iperf(data_by_hops)

plot_runtime_bandwidth_vs_hops(data_by_rate)
