#include "process-application.h"
#include "ns3/assert.h"
#include "ns3/log.h"
#include "ns3/process-manager.h"

namespace ns3 {

NS_LOG_COMPONENT_DEFINE ("ProcessApplication");
NS_OBJECT_ENSURE_REGISTERED (ProcessApplication);

TypeId 
ProcessApplication::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::ProcessApplication")
    .SetParent<Application> ()
    .AddConstructor<ProcessApplication> ()
    ;
  return tid;
}

ProcessApplication::ProcessApplication ()
  : m_stackSize (0),
    m_pid (0)
{}
ProcessApplication::~ProcessApplication ()
{}

void
ProcessApplication::DoDispose (void)
{
  NS_LOG_FUNCTION (this);
  Application::DoDispose ();
}
void 
ProcessApplication::SetBinary (std::string filename)
{
  m_filename = filename;
}
void 
ProcessApplication::SetStackSize (uint32_t stackSize)
{
  m_stackSize = stackSize;
}
void 
ProcessApplication::SetArguments (std::vector<std::string> args)
{
  m_args = args;
}

void 
ProcessApplication::SetEnvironment (std::vector<std::pair<std::string,std::string> > envs)
{
  m_envs = envs;
}

void 
ProcessApplication::StartApplication (void)
{
  NS_LOG_FUNCTION (this);

  Ptr<Node> node = GetNode ();
  Ptr<ProcessManager> manager = node->GetObject<ProcessManager> ();
  if (manager == 0)
    {
      NS_FATAL_ERROR ("You forgot to aggregate a ProcessManager to node=" << node->GetId ());
    }
  if (m_stackSize != 0)
    {
      m_pid = manager->Start (m_filename, m_stackSize, m_args, m_envs);
    } 
  else
    {
      m_pid = manager->Start (m_filename, m_args, m_envs);
    }
}
void 
ProcessApplication::StopApplication (void)
{
  NS_LOG_FUNCTION (this);
  Ptr<Node> node = GetNode ();
  Ptr<ProcessManager> manager = node->GetObject<ProcessManager> ();
  manager->Stop (m_pid);
}


} // namespace ns3
