#ifndef PROCESS_APPLICATION_H
#define PROCESS_APPLICATION_H

#include "ns3/application.h"

namespace ns3 {

/**
 * \brief a wrapper around a posix synchronous application.
 *
 * The sole purpose of this class is to allow users to manipulate
 * these posix synchronous applications in their scripts as if they
 * were normal ns-3 applications and to start them at a specific
 * time using the normal ApplicationContainer::Start method.
 *
 * To implement the Stop method, we send a SIGKILL signal to the
 * target application. Note that doing this will make it impossible
 * to deal correctly with the application's memory management. i.e.,
 * once you call Stop on such an application, valgrind will most
 * likely start reporting a lot of lost memory. Crashes might ensue.
 */
class ProcessApplication : public Application
{
public:
  static TypeId GetTypeId (void);

  ProcessApplication ();
  virtual ~ProcessApplication ();

  void SetBinary (std::string filename);
  void SetStackSize (uint32_t stackSize);
  void SetArguments (std::vector<std::string> args);
  void SetEnvironment (std::vector<std::pair<std::string,std::string> > envs);

private:
  // inherited from Application base class.
  virtual void StartApplication (void);
  virtual void StopApplication (void);
  virtual void DoDispose (void);

  std::string m_filename;
  uint32_t m_stackSize;
  std::vector<std::string> m_args;
  std::vector<std::pair<std::string,std::string> > m_envs;
  uint16_t m_pid;
};

} // namespace ns3

#endif /* PROCESS_APPLICATION_H */
