#ifndef ELF_FILE_READER_H
#define ELF_FILE_READER_H

#include <stdint.h>
#include <string>
#include "elf-types.h"
#include "file-reader.h"

namespace mdbg {

class ElfFileReader
{
public:
  ElfFileReader (const uint8_t *buffer, uint64_t size);
  bool IsOk (void) const;
  uint64_t GetOffset (void) const;
  void Seek (uint64_t offset);
  void Skip (uint64_t offset);
  ElfHalf ReadHalf (void);
  ElfWord ReadWord (void);
  ElfXword ReadXword (void);
  ElfSxword ReadSxword (void);
  ElfOff ReadOff (void);
  ElfAddr ReadAddr (void);
  ElfSword ReadSword (void);
  std::string ReadString (void);
  struct ElfSectionHeader ReadSectionHeader (void);
  struct ElfSectionHeader ReadSectionHeader (uint64_t shn);
  struct ElfProgramHeader ReadProgramHeader (void);
  struct ElfProgramHeader ReadProgramHeader (uint64_t phn);

  struct ElfHeader GetHeader (void);
private:
  void Initialize (void);
  FileReader m_reader;
  struct ElfHeader m_header;
  bool m_64;
};

} // namespace mdbg

#endif /* ELF_FILE_READER_H */
