#ifndef FILE_READER_H
#define FILE_READER_H

#include <stdint.h>
#include <string>

namespace mdbg {

class FileReader
{
public:
  FileReader (const uint8_t *buffer, uint64_t size);
  ~FileReader ();

  void SetFail (void);
  bool IsOk (void) const;
  void SetMsb (void);
  void SetLsb (void);
  uint8_t ReadU8 (void);
  uint16_t ReadU16 (void);
  uint32_t ReadU32 (void);
  uint64_t ReadU64 (void);
  int8_t ReadS8 (void);
  int16_t ReadS16 (void);
  int32_t ReadS32 (void);
  int64_t ReadS64 (void);

  uint64_t GetOffset (void) const;
  void Seek (uint64_t offset);
  void Skip (uint64_t offset);

private:
  bool HasLeft (uint8_t n) const;

  const uint8_t *m_buffer;
  uint64_t m_size;
  uint64_t m_current;
  bool m_lsb;
  bool m_ok;
};

} // namespace mdbg

#endif /* FILE_READER_H */
