#include "mmaped-file.h"
#include "ns3/assert.h"
#include "ns3/log.h"
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

namespace mdbg {

NS_LOG_COMPONENT_DEFINE ("MmapedFile");

MmapedFile::MmapedFile (std::string filename)
{
  m_fd = ::open (filename.c_str (), 0);
  if (m_fd == -1)
    {
      NS_LOG_DEBUG ("unable to open file=" << filename << " error=" << strerror (errno));
      m_ok = false;
      return;
    }
  struct stat st;
  int retval = ::fstat(m_fd, &st);
  if (retval != 0)
    {
      NS_LOG_DEBUG ("unable to fstat file=" << filename << " error=" << strerror (errno));
      m_ok = false;
      return;
    }
  m_size = st.st_size;
  m_buffer = (uint8_t *) ::mmap (0, m_size, PROT_READ, MAP_PRIVATE, m_fd, 0);
  if (m_buffer == 0)
    {
      NS_LOG_DEBUG ("unable to mmap file=" << filename << " error=" << strerror (errno));
      m_ok = false;
      return;
    }
  m_ok = true;
}
MmapedFile::~MmapedFile ()
{
  if (m_buffer != 0)
    {
      int retval;
      retval = ::munmap (m_buffer, m_size);
      NS_ASSERT (retval == 0);
    }
  if (m_fd != -1)
    {
      int retval;
      retval = ::close (m_fd);
      NS_ASSERT (retval == 0);
    }
}

bool
MmapedFile::IsOk (void) const
{
  return m_ok;
}

uint64_t 
MmapedFile::GetSize (void) const
{
  return m_size;
}
uint8_t *
MmapedFile::GetBuffer (void) const
{
  return m_buffer;
}

} // namespace mdbg
