#ifndef LINUX_SOCKET_FD_FACTORY_H
#define LINUX_SOCKET_FD_FACTORY_H

#include "socket-fd-factory.h"
#include <sys/socket.h>
#include <vector>
#include "ns3/net-device.h"

extern "C" {
struct SimExported;
struct SimDevice;
struct SimSocket;
struct SimTask;
}

namespace ns3 {

class Loader;
class NetDevice;
class Task;
class TaskManager;
class Packet;

class LinuxSocketFdFactory : public SocketFdFactory
{
public:
  static TypeId GetTypeId (void);
  LinuxSocketFdFactory ();
  ~LinuxSocketFdFactory ();
  virtual UnixFd *CreateSocket (int domain, int type, int protocol);
  struct SimExported *PeekExported (void);



private:
  friend class LinuxSocketFd;

  // inherited from Object
  virtual void DoDispose (void);
  // called from LinuxSocketFd
  int Close (struct SimSocket *socket);
  ssize_t Recvmsg(struct SimSocket *socket, struct msghdr *msg, int flags);
  ssize_t Sendmsg(struct SimSocket *socket, const struct msghdr *msg, int flags);
  int Getsockname(struct SimSocket *socket, struct sockaddr *name, socklen_t *namelen);
  int Getpeername(struct SimSocket *socket, struct sockaddr *name, socklen_t *namelen);
  int Bind (struct SimSocket *socket, const struct sockaddr *my_addr, socklen_t addrlen);
  int Connect (struct SimSocket *socket, const struct sockaddr *my_addr, socklen_t addrlen);
  int Listen (struct SimSocket *socket, int backlog);
  int Shutdown(struct SimSocket *socket, int how);
  int Accept (struct SimSocket *socket, struct sockaddr *my_addr, socklen_t *addrlen);
  int Ioctl (struct SimSocket *socket, int request, char *argp);
  int Fcntl (struct SimSocket *socket, int cmd, unsigned long arg);
  int Setsockopt (struct SimSocket *socket, int level, int optname,
		  const void *optval, socklen_t optlen);
  int Getsockopt (struct SimSocket *socket, int level, int optname,
		  void *optval, socklen_t *optlen);
  bool CheckFd (struct SimSocket *socket);

  // inherited from Object.
  virtual void NotifyNewAggregate (void);
  // called from Node
  void NotifyAddDevice (Ptr<NetDevice> device);
  // called during initialization with a task context
  // to enter the kernel functions.
  void StartInitializationTask (void);
  static void InitializeStackTrampoline (void *context);
  void InitializeStack (void);
  static LinuxSocketFdFactory *Self (void);
  static struct SimTask *TaskStart (void (*callback) (void *), void *context);
  struct SimTask *DoTaskStart (void (*callback) (void *), void *context);
  static void DevXmit (struct SimDevice *dev, unsigned char *data, int len);
  void DoDevXmit (struct SimDevice *dev, unsigned char *data, int len);
  void RxFromDevice (Ptr<NetDevice> device, Ptr<const Packet> p,
		     uint16_t protocol, const Address & from,
		     const Address &to, NetDevice::PacketType type);

  std::vector<std::pair<Ptr<NetDevice>,struct SimDevice *> > m_devices;
  std::string m_library;
  Loader *m_loader;
  struct SimExported *m_exported;
  std::vector<Task *> m_kernelTasks;
  Ptr<TaskManager> m_manager;
  int m_flags;
};

} // namespace ns3

#endif /* LINUX_SOCKET_FD_FACTORY_H */
