#include "sys/simu-stat.h"
#include "utils.h"
#include "process.h"
#include "ns3/log.h"
#include "ns3/assert.h"
#include <errno.h>

using namespace ns3;

NS_LOG_COMPONENT_DEFINE ("SimuStat");

int simu_xstat (int ver, const char *path, struct stat *buf)
{
  Thread *current = Current ();
  NS_LOG_FUNCTION (current << UtilsGetNodeId () << path << buf);
  NS_ASSERT (current != 0);
  if (std::string (path) == "")
    {
      current->err = ENOENT;
      return -1;
    }
  int retval = ::__xstat (ver, UtilsGetRealFilePath (path).c_str (), buf);
  if (retval == -1)
    {
      current->err = errno;
      return -1;
    }
  return retval;
}
int simu_xstat64 (int ver, const char *path, struct stat64 *buf)
{
  Thread *current = Current ();
  NS_LOG_FUNCTION (current << UtilsGetNodeId () << path << buf);
  NS_ASSERT (current != 0);
  if (std::string (path) == "")
    {
      current->err = ENOENT;
      return -1;
    }
  int retval = ::__xstat64 (ver, UtilsGetRealFilePath (path).c_str (), buf);
  if (retval == -1)
    {
      current->err = errno;
      return -1;
    }
  return retval;
}
int simu_fxstat (int ver, int fd, struct stat *buf)
{
  Thread *current = Current ();
  NS_LOG_FUNCTION (current << UtilsGetNodeId () << fd);
  NS_ASSERT (current != 0);
  int index = UtilsSearchOpenFd (fd);
  if (index == -1)
    {
      current->err = EBADF;
      return -1;
    }
  UnixFd *unixFd = current->process->openFiles[index].second;
  int retval = unixFd->Fxstat (ver, buf);
  return retval;
}
int simu_fxstat64 (int ver, int fd, struct stat64 *buf)
{
  Thread *current = Current ();
  NS_LOG_FUNCTION (current << UtilsGetNodeId () << fd);
  NS_ASSERT (current != 0);
  int index = UtilsSearchOpenFd (fd);
  if (index == -1)
    {
      current->err = EBADF;
      return -1;
    }
  UnixFd *unixFd = current->process->openFiles[index].second;
  int retval = unixFd->Fxstat64 (ver, buf);
  return retval;
}
int simu_lxstat (int ver, const char *pathname, struct stat *buf)
{
  Thread *current = Current ();
  NS_LOG_FUNCTION (current << UtilsGetNodeId () << pathname << buf);
  NS_ASSERT (current != 0);
  if (std::string (pathname) == "")
    {
      current->err = ENOENT;
      return -1;
    }
  int retval = ::__lxstat (ver, UtilsGetRealFilePath (pathname).c_str (), buf);
  if (retval == -1)
    {
      current->err = errno;
      return -1;
    }
  return retval;
}
int simu_lxstat64 (int ver, const char *pathname, struct stat64 *buf)
{
  Thread *current = Current ();
  NS_LOG_FUNCTION (current << UtilsGetNodeId () << pathname << buf);
  NS_ASSERT (current != 0);
  if (std::string (pathname) == "")
    {
      current->err = ENOENT;
      return -1;
    }
  int retval = ::__lxstat64 (ver, UtilsGetRealFilePath (pathname).c_str (), buf);
  if (retval == -1)
    {
      current->err = errno;
      return -1;
    }
  return retval;
}
